@@ -4,11 +4,19 @@
 # constructor is distributed under the terms of the BSD 3-clause license.
 # Consult LICENSE.txt or http://opensource.org/licenses/BSD-3-Clause.
 
+from functools import partial
+from os.path import abspath, dirname
 import re
 import sys
-from os.path import abspath
 
-import yaml
+try:
+    import yaml
+except:
+    import ruamel_yaml as yaml
+
+from constructor.exceptions import (
+    UnableToParse, UnableToParseMissingJinja2, YamlParsingError,
+)
 
 
 PREABLE = '''\n
@@ -47,6 +55,19 @@ key below, but also when using the `packages` key ,unless the full URL is
 given in the `packages` list (see below).
 '''),
 
+    ('channels_remap',         False, list, '''
+List of (src, dest) channels, from which, channels from 'src' are also
+considered while running solver, but are replaced by corresponding values from
+dest when writing urls{,.txt}. Example use:
+channels_remap:
+  -
+      src: file:///tmp/a3/conda-bld
+      dest: https://repo.continuum.io/pkgs/main
+  -
+      src: file:///tmp/r/conda-bld
+      dest: https://repo.continuum.io/pkgs/r
+'''),
+
     ('specs',                  False, (list, str), '''
 List of package specifications, e.g. `python 2.7*`, `pyzmq` or `numpy >=1.8`.
 This list of specifications if given to the conda resolver (as if you were
@@ -71,6 +92,11 @@ Optionally, the MD5 hash sum of the package, may be added after an immediate
 Packages for menu items will be installed (if the conda package contains the
 necessary metadata in "Menu/<package name>.json").  Menu items are currently
 only supported on Windows.  By default, all menu items will be installed.
+'''),
+
+    ('ignore_duplicate_files',  False, bool, '''
+By default, constructor will error out when adding packages with duplicate
+files in them. Enable this option to warn instead and continue.
 '''),
 
     ('install_in_dependency_order', False, bool, '''
@@ -88,6 +114,11 @@ of the created installer (if it includes conda).
     ('installer_filename',     False, str, '''
 The filename of the installer being created.  A reasonable default filename
 will determined by the `name`, `version`, platform and installer type.
+'''),
+
+    ('installer_type',     False, str, '''
+The type of the installer being created.  Possible values are "sh", "pkg",
+and "exe".  By default, the type is "sh" on Unix, and "exe" on Windows.
 '''),
 
     ('license_file',           False, str, '''
@@ -99,6 +130,29 @@ process.
 By default, no conda packages are preserved after running the created
 installer in the `pkgs` directory.  Using this option changes the default
 behavior.
+'''),
+
+    ('signing_identity_name',  False, str, '''
+By default, the MacOS pkg installer isn't signed. If an identity name is specified
+using this option, it will be used to sign the installer. Note that you will need
+to have a certificate and corresponding private key together called an 'identity'
+in one of your accessible keychains.
+'''),
+
+    ('attempt_hardlinks',          False, bool, '''
+By default, conda packages are extracted into the root environment and then
+patched. Enabling this option will result into extraction of the packages into
+the `pkgs` directory and the files in the root environment will be hardlinks to
+the files kept in the `pkgs` directory and then patched accordingly.
+'''),
+
+    ('write_condarc',          False, bool, '''
+If set, a .condarc file is written to the root of the enviornment if there are
+any channels or conda_default_channels set.
+'''),
+
+    ('company',                False, str, '''
+Name of the company/entity who is responsible for the installer.
 '''),
 
     ('pre_install',            False, str, '''
@@ -130,6 +184,28 @@ Like `welcome_image` for Windows, re-sized to 256 x 256 pixels.
 The color of the default images (when not providing explicit image files)
 used on Windows.  Possible values are `red`, `green`, `blue`, `yellow`.
 The default is `blue`.
+'''),
+
+    ('welcome_image_text',     False, str, '''
+If `welcome_image` is not provided, use this text when generating the image
+(Windows only). Defaults to `name`.
+'''),
+
+    ('header_image_text',      False, str, '''
+If `header_image` is not provided, use this text when generating the image
+(Windows only). Defaults to `name`.
+'''),
+
+    ('add_to_path_default',    False, bool, '''
+Default choice for whether to add the installation to the PATH environment
+variable. The user is still able to change this during interactive
+installation.
+'''),
+
+    ('register_python_default',  False, bool, '''
+Default choice for whether to register the installed Python instance as the
+system's default Python. The user is still able to change this during
+interactive installation. (Windows only)
 '''),
 ]
 
@@ -141,6 +217,7 @@ def ns_platform(platform):
         linux32 = bool(p == 'linux-32' or p == 'linux-armv7l'),
         linux64 = bool(p == 'linux-64' or p == 'linux-ppc64le'),
         armv7l = bool(p == 'linux-armv7l'),
+        aarch64 = bool(p == 'aarch64'),
         ppc64le = bool(p == 'linux-ppc64le'),
         x86 = p.endswith(('-32', '-64')),
         x86_64 = p.endswith('-64'),
@@ -167,13 +244,34 @@ def select_lines(data, namespace):
     return '\n'.join(lines) + '\n'
 
 
+# adapted from conda-build
+def yamlize(data, directory, content_filter):
+    data = content_filter(data)
+    try:
+        return yaml.load(data)
+    except yaml.error.YAMLError as e:
+        if ('{{' not in data) and ('{%' not in data):
+            raise UnableToParse(original=e)
+        try:
+            from constructor.jinja import render_jinja
+        except ImportError as ex:
+            raise UnableToParseMissingJinja2(original=ex)
+        data = render_jinja(data, directory, content_filter)
+        return yaml.load(data)
+
+
 def parse(path, platform):
     try:
         with open(path) as fi:
             data = fi.read()
     except IOError:
         sys.exit("Error: could not open '%s' for reading" % path)
-    res = yaml.load(select_lines(data, ns_platform(platform)))
+    directory = dirname(path)
+    content_filter = partial(select_lines, namespace=ns_platform(platform))
+    try:
+        res = yamlize(data, directory, content_filter)
+    except YamlParsingError as e:
+        sys.exit(e.error_msg())
 
     try:
         res['version'] = str(res['version'])
